// Fill out your copyright notice in the Description page of Project Settings.


#include "BountyDirector.h"

#include "Components/ArrowComponent.h"

// Sets default values
ABountyDirector::ABountyDirector()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

}

// Called every frame
void ABountyDirector::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

}
// Called when the game starts or when spawned
void ABountyDirector::BeginPlay()
{
	Super::BeginPlay();

	PlayerChar = Cast<AEndlessVendettaCharacter>(GetWorld()->GetFirstPlayerController()->GetCharacter());
	if (!IsValid(PlayerChar)) UE_LOG(LogTemp, Fatal, TEXT("Players Character isn't being loaded in BeginPlay() in BountyDirector"));

	// Setup a component for bounties to attach to on the player
	for (UActorComponent* ActorComp : PlayerChar->GetComponentsByTag(UArrowComponent::StaticClass(), FName("Bounty")))
	{
		BountyAttachmentPoint = Cast<UArrowComponent>(ActorComp);
		if (!IsValid(BountyAttachmentPoint)) UE_LOG(LogTemp, Fatal, TEXT("There's no Bounty Attachment UArrowComponent on Players Char "));
		break;
	}

	// Setup up starting Side Bounty indexes
	for (int Index = 0; Index < 3; Index++)
	{
		CurrentSideBountyIndexes.Add(Index);
	}

	ActiveSideBounties.SetNum(3);
	SpawnBountyAndItsSideBounties();
	UpdateBountyDisplay();
}

void ABountyDirector::SpawnBountyAndItsSideBounties()
{
	UE_LOG(LogTemp, Warning, TEXT("Spawning Bounty..."));
	
	if (BountyClassArray.IsEmpty()) return;
 	if (!IsValid(BountyClassArray[CurrentBountyIndex])) UE_LOG(LogTemp, Fatal, TEXT("Missing Bounty at Index: %d"), CurrentBountyIndex);
	
	// Spawn Main Bounty which the current bounty index is pointing to
	FActorSpawnParameters SpawnParams;
	SpawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	
	AActor* BountyActor = GetWorld()->SpawnActor<AActor>(BountyClassArray[CurrentBountyIndex], PlayerChar->GetActorLocation(), PlayerChar->GetActorRotation(), SpawnParams);
	const FAttachmentTransformRules AttachmentTransformRules(EAttachmentRule::SnapToTarget, true);
	BountyActor->AttachToComponent(BountyAttachmentPoint, AttachmentTransformRules);

	ActiveBounty = Cast<ABountyClass>(BountyActor);
	if (!IsValid(ActiveBounty)) UE_LOG(LogTemp, Fatal, TEXT("Failed to Cast to Bounty class"));

	// WHEN MARCEL STOPS PLAYING WITH THE CHARACTER, ADD THIS BOUNTY TO PLAYERS ARRAY OF ACTIVE BOUNTIES!!!!!
	ActiveBounty->CompletedFirstCheckpoint.AddDynamic(this, &ABountyDirector::DestroyActiveSideBounties);

	// Spawn in Side Bounties which are linked to the Main Bounty
	int ActiveSideBountyIndex = 0;
	for (int CurrentSideBountyIndex : CurrentSideBountyIndexes)
	{
		if (SideBountyClassArray.Num() <= CurrentSideBountyIndex || !IsValid(SideBountyClassArray[CurrentSideBountyIndex])) continue;

		AActor* SideBountyActor = GetWorld()->SpawnActor<AActor>(SideBountyClassArray[CurrentSideBountyIndex], PlayerChar->GetActorLocation(), PlayerChar->GetActorRotation(), SpawnParams);
		ASideBountyClass* SideBounty = Cast<ASideBountyClass>(SideBountyActor);
		SideBounty->AttachToComponent(BountyAttachmentPoint, AttachmentTransformRules);

		ActiveSideBounties[ActiveSideBountyIndex] = SideBounty;
		SideBounty->ActiveSBC_Index = ActiveSideBountyIndex;
		ActiveSideBounties[ActiveSideBountyIndex]->CompletedSideBounty.AddDynamic(this, &ABountyDirector::SideBountyCompleted);
		// ONCE AGAIN WHEN MARCEL STOPS PLAYING WITH THE CHARACTER, ADD THIS SIDE BOUNTY TO PLAYERS ARRAY OF ACTIVE BOUNTIES!!!!!
		ActiveSideBountyIndex++;
	}
}

void ABountyDirector::UpdateBountyProgression()
{
	if (!IsValid(ActiveBounty)) return;

	if (ActiveBounty->IsCompleted()) FinishActiveBounty();
}

void ABountyDirector::FinishActiveBounty()
{
	ActiveBounty->CollectRewards();

	// Destroy old Bounties
	ActiveBounty->DestroyReturnToShipStep();
	ActiveBounty->Destroy();
	DestroyActiveSideBounties();

	// Increment Main and Side Bounty Indexes
	CurrentBountyIndex++;
	for (int i = 0; i < 3; i++)
	{
		CurrentSideBountyIndexes[i] += 3;
	}

	// Game Completion Check
	if (CurrentBountyIndex >= BountyClassArray.Num())
	{
		UE_LOG(LogTemp, Warning, TEXT("Finished all bounties currently available in the game :)"));
		ActiveBounty = nullptr;
		UpdateBountyDisplay();
		return;
	}

	SpawnBountyAndItsSideBounties();
	UpdateBountyDisplay();
}

void ABountyDirector::DestroyActiveSideBounties()
{
	for (ASideBountyClass* SideBounty : ActiveSideBounties)
	{
		if (!IsValid(SideBounty)) continue;

		// WHEN MARCEL STOPS PLAYING WITH PLAYERS CHARACTER, REMOVE THIS SIDE BOUNTY FROM ACTIVE BOUNTIES

		SideBounty->DestroyCheckpoints();
		SideBounty->Destroy();
	}

	UpdateBountyDisplay();
}

void ABountyDirector::SideBountyCompleted(int SideBountyIndex)
{
	UE_LOG(LogTemp,Warning,TEXT("Updating Main Bounties Checkpoints"));
	ActiveBounty->UpdateBountyCheckpoints(ActiveSideBounties[SideBountyIndex]->GetReplacementCheckpoints());

	// WHEN MARCEL STOPS PLAYING WITH PLAYER CHAR, REMOVE THIS SIDE BOUNTY FROM ACTIVE BOUNTIES
	ActiveSideBounties[SideBountyIndex]->CollectRewards();
	ActiveSideBounties[SideBountyIndex]->DestroyCheckpoints();
	ActiveSideBounties[SideBountyIndex]->Destroy();

	UpdateBountyDisplay();
}


